/********************************************************************/
/* PURPOSE:       Return the number of specified observations in a dataset
/* INPUT:
/*                DSN = Name of permanent or temporary dataset
/*                WHERE = The condition to subset the dataset 
/*                
/* OUTPUT:  	  Returns the number of observations in a dataset
/* MACROS USED:   varexst
/* EXAMPLES:
/*		 %let NUMOBS = %nobs(perm.status);
/*		 %if %nobs(arv2) > 0 %then %do;
/*	       	 title "There are %nobs(mydata) observations in mydata";
/*		 %nobs(DSN=perm.txw0120, WHERE=dosest=1)
/*
/* NOTES:	 This macro has a DISTINCT parmameter at our 
/*		 organization and works via a UNIX script.  Feel
/* 		 free to add this option to this macro for your own
/*		 use.
/********************************************************************/

%macro nobs(DSN, WHERE);
    %local LIBREF PATH MEMORIG WORK  DSID RC WHERE NOBS RANNUM RANINT;
    
    /* If no dataset is given, use the one most recently created. */
    %if %length(&DSN) = 0 %then %do;
        %if &SYSLAST = _NULL_ %then %do;
            %put %upcase(error): The parameter DSN cannot be missing if SYSLAST is _NULL_.;
            %return;
        %end;
        %let DSN = &SYSLAST;
    %end;
    %else %do;
        %let DSN = %upcase(&DSN);

        /* Remove any parentheses after the dataset name */
        %if %index(&DSN, %str(%()) %then %do;
            %let DSN = %scan(&DSN, 1, %str(%());
            %put %upcase(note: (cbar)): Parenthetical clause attached to &DSN will be ignored.;
        %end;
    %end;

    /* Create two-level name for DSN and make sure it exists */
    %if %index(&DSN, .) = 0 %then %let DSN = WORK.&DSN;

    %if %sysfunc(exist(&DSN)) = 0 %then %do;
        %put %upcase(error): File &DSN..DATA does not exist.;
        %return;
    %end;

    %if %length(&WHERE) > 0 %then %let WHERE = (where=(&WHERE));

    /* Open datset and subset if requested */
    %let DSID = %sysfunc( open(&DSN&WHERE, i) );

    /* Return error and exit if unable to open dataset */
    %if &DSID = 0 %then %do;
        %put %upcase(error): Could not count the observations in &DSN..;
        %return;

    /* Read and return the number of observations */
    %if %length(&WHERE) > 0 %then
        %let NOBS = %sysfunc( attrn(&DSID, NLOBSF) );
    %else
        %let NOBS = %sysfunc( attrn(&DSID, NLOBS) );&NOBS 

    /* Close the dataset */
    %let RC = %sysfunc( close(&DSID) );
    
%mend nobs;
