/************************************************/
/* PURPOSE:       Checks if variables exist in the dataset. 
/* INPUT:
/*                DATA   = Dataset name
/*                VARS   = Variables to check in dataset
/*                ADDMSG = Text to append to error message if
/*                         variable not in dataset 
/*                IFERR  = ENDSAS - to end SAS session if variable 
/*                         doesn't exist in dataset
/*                
/* OUTPUT:  	  Warning message if variable(s) don't exist
/* MACROS USED:   NA
/*   
/* NOTES:         __PARMERR is a macrovariable created and used by
/*                other programs that call this macro. Permission has been
/*                granted by Head of CBAR Programming Core to not define
/*                this macrovariable as local. 
/*
/* EXAMPLES:      %varexst(temp, var1); 
/*                %varexst(perm.dsn, var1 var2, Adl text, ENDSAS); 
/*
/************************************************/

%macro varexst(DATA, VARS, ADDMSG, IFERR);

  /* [JB20170605] Added local variables WARNVAR and WARN2VAR */
  %local I RC DSID THISVAR VARNUM WARNVAR WARN2VAR;
  %let I=1;
  %let THISVAR=%scan(&VARS,&I,%str( ));
  %let DSID = %sysfunc(open(&DATA));
  %if &DSID %then %do;
    /* check that each variable provided in VARS list exists in the dataset DATA */ 
    %do %while (&THISVAR ne %str( ));
      /* get variable's position number in the dataset, 0 if not in dataset */ 
      %let VARNUM=%sysfunc(varnum(&DSID, &THISVAR));
      /* return warning message if variable not in dataset */ 
      %if &VARNUM=0 %then %do;
        %put %upcase(warning): Variable &THISVAR required on dataset &DATA &ADDMSG;
          /* [JB20170605] Removed previous line prompting ENDSAS and added in a let
             statement to give WARNVAR the value of 1 if a warning was generated*/
        %let WARNVAR=1;
        %let __PARMERR=1;
      %end;
      %let I=%eval(&I+1);
      %let THISVAR=%scan(&VARS,&I,%str( ));
    %end; 
    %let RC=%sysfunc(close(&DSID));
  %end;
  %else %do;
    %put %upcase(warning): Unable to open dataset &DATA in varexst macro;
    /* [JB20170605] Gave WARN2VAR the value of 1 when the dataset warning is prompted*/
    %let WARN2VAR=1;
  %end;
      /* [JB20170605] Created a new condition that will prompt the ENDSAS
         when specified once the entire list of variables has been looped through*/
      %if &WARNVAR > 0 and %upcase(&IFERR)=ENDSAS %then %do;   
         endsas;   
      %end;
      /* [JB20170605] Created a new condition that will prompt the ENDSAS when the dataset doesn't exist*/
      %if &WARN2VAR > 0 %then %do;   
         endsas;   
      %end;
            
%mend;



